from ...interface import IPacker


class OrderDataPacker(IPacker):
    def __init__(self, obj) -> None:
        super().__init__(obj)

    def obj_to_tuple(self):
        return [str(self._obj.InvestorID), str(self._obj.BrokerID), str(self._obj.ExchangeID), str(self._obj.ProductID),
                int(self._obj.ProductType), str(self._obj.InstrumentID), str(self._obj.OrderTime), str(self._obj.CancelTime),
                str(self._obj.TradingDay), str(self._obj.InsertDate), str(self._obj.UpdateTime), str(self._obj.StatusMsg),
                int(self._obj.FrontID), int(self._obj.SessionID), str(self._obj.OrderRef), str(self._obj.OrderLocalNo),
                str(self._obj.OrderID), str(self._obj.RelativeOrderSysID), str(self._obj.BrokerOrderSeq), float(self._obj.Price),
                float(self._obj.StopPrice), int(self._obj.Volume), int(self._obj.NoTradedVolume), int(self._obj.Status),
                int(self._obj.Direction), int(self._obj.OpenCloseType), int(self._obj.PriceCond), int(self._obj.TimeCond),
                int(self._obj.VolumeCond), int(self._obj.HedgeType), int(self._obj.OrderType), int(self._obj.ActionType),
                int(self._obj.ContingentCond), float(self._obj.FrozenMarginPrice), float(self._obj.FrozenMargin), float(self._obj.FrozenCommission),
                int(self._obj.ShowVolume), int(self._obj.MinVolume), (self._obj.PricePrecision), str(self._obj.FinBizNo),
                str(self._obj.FinAlgoNo), int(self._obj.FinInsertLocalTime), int(self._obj.FinRtnOrderLocalTime), str(self._obj.FinLockNo),
                str(self._obj.BizName), int(self._obj.RtnOrderLocalTime)]

    def tuple_to_obj(self, t):
        if len(t) >= 46:
            self._obj.InvestorID = t[0]
            self._obj.BrokerID = t[1]
            self._obj.ExchangeID = t[2]
            self._obj.ProductID = t[3]
            self._obj.ProductType = t[4]
            self._obj.InstrumentID = t[5]
            self._obj.OrderTime = t[6]
            self._obj.CancelTime = t[7]
            self._obj.TradingDay = t[8]
            self._obj.InsertDate = t[9]
            self._obj.UpdateTime = t[10]
            self._obj.StatusMsg = t[11]
            self._obj.FrontID = t[12]
            self._obj.SessionID = t[13]
            self._obj.OrderRef = t[14]
            self._obj.OrderLocalNo = t[15]
            self._obj.OrderID = t[16]
            self._obj.RelativeOrderSysID = t[17]
            self._obj.BrokerOrderSeq = t[18]
            self._obj.Price = t[19]
            self._obj.StopPrice = t[20]
            self._obj.Volume = t[21]
            self._obj.NoTradedVolume = t[22]
            self._obj.Status = t[23]
            self._obj.Direction = t[24]
            self._obj.OpenCloseType = t[25]
            self._obj.PriceCond = t[26]
            self._obj.TimeCond = t[27]
            self._obj.VolumeCond = t[28]
            self._obj.HedgeType = t[29]
            self._obj.OrderType = t[30]
            self._obj.ActionType = t[31]
            self._obj.ContingentCond = t[32]
            self._obj.FrozenMarginPrice = t[33]
            self._obj.FrozenMargin = t[34]
            self._obj.FrozenCommission = t[35]
            self._obj.ShowVolume = t[36]
            self._obj.MinVolume = t[37]
            self._obj.PricePrecision = t[38]
            self._obj.FinBizNo = t[39]
            self._obj.FinAlgoNo = t[40]
            self._obj.FinInsertLocalTime = t[41]
            self._obj.FinRtnOrderLocalTime = t[42]
            self._obj.FinLockNo = t[43]
            self._obj.BizName = t[44]
            self._obj.RtnOrderLocalTime = t[45]

            return True
        return False



from typing import List
from ...interface import IData
from .ohlc_data import OHLCData
from ...packer.market.history_ohlc_param_data_packer import HistoryOHLCParamDataPacker


class HistoryOHLCParamData(IData):
    def __init__(self, market_name: str = '', exchange_id: str = '', instrument_id: str = '', range: str = "60",
                 start_date: str = '', end_date: str = '', ohlc_list: list = '', is_return_list: bool = False):
        super().__init__(HistoryOHLCParamDataPacker(self))
        self._MarketName: str = market_name
        self._ExchangeID: str = exchange_id
        self._InstrumentID: str = instrument_id
        self._Range: str = range
        self._StartDate: str = start_date
        self._EndDate: str = end_date
        self._OHLCList: list = ohlc_list
        self._IsReturnList: bool = is_return_list

    @property
    def MarketName(self):
        return self._MarketName

    @MarketName.setter
    def MarketName(self, value: str):
        self._MarketName = value

    @property
    def ExchangeID(self):
        return self._ExchangeID

    @ExchangeID.setter
    def ExchangeID(self, value: str):
        self._ExchangeID = value

    @property
    def InstrumentID(self):
        return self._InstrumentID

    @InstrumentID.setter
    def InstrumentID(self, value: str):
        self._InstrumentID = value

    @property
    def Range(self):
        return self._Range

    @Range.setter
    def Range(self, value: str):
        self._Range = value

    @property
    def StartDate(self):
        return self._StartDate

    @StartDate.setter
    def StartDate(self, value: str):
        self._StartDate = value

    @property
    def EndDate(self):
        return self._EndDate

    @EndDate.setter
    def EndDate(self, value: str):
        self._EndDate = value

    @property
    def OHLCList(self):
        return self._OHLCList

    @OHLCList.setter
    def OHLCList(self, value: List[OHLCData]):
        self._OHLCList = value

    @property
    def IsReturnList(self):
        return self._IsReturnList

    @IsReturnList.setter
    def IsReturnList(self, value: bool):
        self._IsReturnList = value



from typing import Tuple
from .req_rsp import ReqRspDict, ReqRsp
from ..interface import IData, MsgID
from ..tsocket import TSocket
from ..data.message_data import MessageData
from ..data.login_data import LoginData


class BaseHandle():
    __ReqID: int = 0
    __ReqRspDict: ReqRspDict = ReqRspDict()

    def __init__(self, tsocket: TSocket):
        self.__TSocket = tsocket
        self.__TSocket.set_base_callback(self.__recv_msg)

    def login(self, login_data: LoginData) -> Tuple[bool, str]:
        return self.__wait_send_msg(int(MsgID.MSGID_Base_Login.value), login_data)

    def __send_heart(self):
        msg = MessageData(mid=int(MsgID.MSGID_Base_Heart.value))
        self.__TSocket.send_message(msg)

    def __recv_msg(self, msg: MessageData):
        if msg.MID == int(MsgID.MSGID_Base_Heart.value):
            self.__send_heart()
            return
        elif msg.MID == int(MsgID.MSGID_Base_SessionInit.value):
            return

        key = '%s_%s' % (msg.MID, msg.RequestID)
        reqrsp: ReqRsp = self.__ReqRspDict.get_reqrsp(key)
        if reqrsp is not None:
            reqrsp.append_rsp(msg)

    def __wait_send_msg(self, mid, params: IData):
        self.__ReqID = self.__ReqID + 1
        msg = MessageData(mid=mid, request_id=self.__ReqID)
        if params is not None:
            msg.UData = params.pack()

        key = '%s_%s' % (mid, self.__ReqID)

        req_rsp = self.__ReqRspDict.new_reqrsp(key, msg)
        if self.__TSocket.send_message(msg) is False:
            self.__ReqRspDict.remove(key)
            return [False, '发送命令失败']
        rsp = req_rsp.wait_last_rsp(10)
        if rsp is None:
            self.__ReqRspDict.remove(key)
            return [False, '发送超时']

        ret = [rsp.RspSuccess, rsp.RspMsg]
        self.__ReqRspDict.remove(key)
        return ret



from ...interface import IData
from ...packer.chart.time_axis_param_data_packer import TimeAxisParamDataPacker


class TimeAxisParamData(IData):
    def __init__(self, show_labels: str = '', format: str = ''):
        """时间轴配置参数

        Args:
            show_labels (str, optional): x轴显示的标签例如 09:00:00;15:00:00 表示时间标签只显示9点和15点. Defaults to ''.
            format (str, optional): 时间显示格式. Defaults to ''.
        """
        super().__init__(TimeAxisParamDataPacker(self))
        self._ShowLabels: str = show_labels
        self._Format: str = format

    @property
    def ShowLabels(self):
        return self._ShowLabels

    @ShowLabels.setter
    def ShowLabels(self, value: str):
        self._ShowLabels = value

    @property
    def Format(self):
        return self._Format

    @Format.setter
    def Format(self, value: str):
        self._Format = value



from ...interface import IPacker


class PlotParamDataPacker(IPacker):
    def __init__(self, obj) -> None:
        super().__init__(obj)

    def obj_to_tuple(self):
        return [int(self._obj.PlotIndex), int(self._obj.Height), (self._obj.IsEqValueAxis), int(self._obj.GridStyle),
                str(self._obj.GridColor), str(self._obj.BackGroundColor), str(self._obj.BorderColor), int(self._obj.BorderWidth),
                bool(self._obj.ShowLegendItem), str(self._obj.PlotName), str(self._obj.PlotNameColor), int(self._obj.GridXStyle),
                str(self._obj.GridXColor)]

    def tuple_to_obj(self, t):
        if len(t) >= 13:
            self._obj.PlotIndex = t[0]
            self._obj.Height = t[1]
            self._obj.IsEqValueAxis = t[2]
            self._obj.GridStyle = t[3]
            self._obj.GridColor = t[4]
            self._obj.BackGroundColor = t[5]
            self._obj.BorderColor = t[6]
            self._obj.BorderWidth = t[7]
            self._obj.ShowLegendItem = t[8]
            self._obj.PlotName = t[9]
            self._obj.PlotNameColor = t[10]
            self._obj.GridXStyle = t[11]
            self._obj.GridXColor = t[12]

            return True
        return False



from ...interface import IData
from ...packer.trade.trade_param_data_packer import TradeParamDataPacker


class TradeParamData(IData):
    def __init__(self, trade_names: str = ''):
        super().__init__(TradeParamDataPacker(self))
        self._TradeNames = trade_names

    @property
    def TradeNames(self):
        return self._TradeNames

    @TradeNames.setter
    def TradeNames(self, value: str):
        self._TradeNames = value



from ...interface import IPacker


class TradeOrderDataPacker(IPacker):
    def __init__(self, obj) -> None:
        super().__init__(obj)

    def obj_to_tuple(self):
        return [str(self._obj.ExchangeID), str(self._obj.ProductID), str(self._obj.InstrumentID), str(self._obj.TradeTime),
                str(self._obj.TradingDay), str(self._obj.TradeDate), str(self._obj.BrokerOrderSeq), str(self._obj.OrderID),
                str(self._obj.TradeID), float(self._obj.Price), int(self._obj.Volume), int(self._obj.UnCloseVolume),
                int(self._obj.Direction), int(self._obj.OpenCloseType), int(self._obj.HedgeType), bool(self._obj.IsYesterdayTrade),
                float(self._obj.CloseProfit), float(self._obj.CurrMargin), float(self._obj.Commission), int(self._obj.TradeType),
                int(self._obj.RtnTradeOrderLocalTime)]

    def tuple_to_obj(self, t):
        if len(t) >= 21:
            self._obj.ExchangeID = t[0]
            self._obj.ProductID = t[1]
            self._obj.InstrumentID = t[2]
            self._obj.TradeTime = t[3]
            self._obj.TradingDay = t[4]
            self._obj.TradeDate = t[5]
            self._obj.BrokerOrderSeq = t[6]
            self._obj.OrderID = t[7]
            self._obj.TradeID = t[8]
            self._obj.Price = t[9]
            self._obj.Volume = t[10]
            self._obj.UnCloseVolume = t[11]
            self._obj.Direction = t[12]
            self._obj.OpenCloseType = t[13]
            self._obj.HedgeType = t[14]
            self._obj.IsYesterdayTrade = t[15]
            self._obj.CloseProfit = t[16]
            self._obj.CurrMargin = t[17]
            self._obj.Commission = t[18]
            self._obj.TradeType = t[19]
            self._obj.RtnTradeOrderLocalTime = t[20]

            return True
        return False



from typing import List
from ...interface import IData
from ..market.ohlc_data import OHLCData
from ...packer.chart.ohlc_value_list_data_packer import OHLCValueListDataPacker


class OHLCValueListData(IData):
    def __init__(self, ohlc_value_list: List[OHLCData]):
        super().__init__(OHLCValueListDataPacker(self))
        self._OHLCList: List[OHLCData] = ohlc_value_list

    @property
    def OHLCList(self):
        return self._OHLCList

    @OHLCList.setter
    def OHLCList(self, value: List[OHLCData]):
        self._OHLCList = value



from typing import List
from ...interface import IData
from ...packer.market.save_ohlc_list_param_data_packer import SaveOHLCListParamDataPacker


class SaveOHLCListParamData(IData):
    def __init__(self, market_name: str = '', exchange_id: str = '', instrument_id: str = '',
                 range: int = 60, trading_day: str = '', pre_settlement_price: float = 0.0,
                 action_day_list: List[str] = [],
                 action_timespan_list: List[int] = [],
                 trading_time_list: List[str] = [],
                 start_time_list: List[str] = [],
                 end_time_list: List[str] = [],
                 total_turnover_list: List[float] = [],
                 open_interest_list: List[float] = [],
                 open_price_list: List[float] = [],
                 open_bid_price_list: List[float] = [],
                 open_ask_price_list: List[float] = [],
                 open_bid_volume_list: List[int] = [],
                 open_ask_volume_list: List[int] = [],
                 high_price_list: List[float] = [],
                 high_bid_price_list: List[float] = [],
                 high_ask_price_list: List[float] = [],
                 high_bid_volume_list: List[int] = [],
                 high_ask_volume_list: List[int] = [],
                 lower_price_list: List[float] = [],
                 lower_bid_price_list: List[float] = [],
                 lower_ask_price_list: List[float] = [],
                 lower_bid_volume_list: List[int] = [],
                 lower_ask_volume_list: List[int] = [],
                 close_price_list: List[float] = [],
                 close_bid_price_list: List[float] = [],
                 close_ask_price_list: List[float] = [],
                 close_bid_volume_list: List[int] = [],
                 close_ask_volume_list: List[int] = []
                 ):
        super().__init__(SaveOHLCListParamDataPacker(self))
        self._MarketName: str = market_name
        self._ExchangeID: str = exchange_id
        self._InstrumentID: str = instrument_id
        self._Range: int = range
        self._TradingDay: str = trading_day
        self._ActionDayList: List[str] = action_day_list
        self._PreSettlementPrice: float = pre_settlement_price
        self._ActionTimespanList: List[int] = action_timespan_list
        self._TradingTimeList: List[str] = trading_time_list
        self._StartTimeList: List[str] = start_time_list
        self._EndTimeList: List[str] = end_time_list
        self._TotalTurnoverList: List[float] = total_turnover_list
        self._OpenInterestList: List[float] = open_interest_list
        self._OpenPriceList: List[float] = open_price_list
        self._OpenBidPriceList: List[float] = open_bid_price_list
        self._OpenAskPriceList: List[float] = open_ask_price_list
        self._OpenBidVolumeList: List[int] = open_bid_volume_list
        self._OpenAskVolumeList: List[int] = open_ask_volume_list

        self._HighPriceList: List[float] = high_price_list
        self._HighBidPriceList: List[float] = high_bid_price_list
        self._HighAskPriceList: List[float] = high_ask_price_list
        self._HighBidVolumeList: List[int] = high_bid_volume_list
        self._HighAskVolumeList: List[int] = high_ask_volume_list

        self._LowerPriceList: List[float] = lower_price_list
        self._LowerBidPriceList: List[float] = lower_bid_price_list
        self._LowerAskPriceList: List[float] = lower_ask_price_list
        self._LowerBidVolumeList: List[int] = lower_bid_volume_list
        self._LowerAskVolumeList: List[int] = lower_ask_volume_list

        self._ClosePriceList: List[float] = close_price_list
        self._CloseBidPriceList: List[float] = close_bid_price_list
        self._CloseAskPriceList: List[float] = close_ask_price_list
        self._CloseBidVolumeList: List[int] = close_bid_volume_list
        self._CloseAskVolumeList: List[int] = close_ask_volume_list

    @property
    def MarketName(self):
        return self._MarketName

    @MarketName.setter
    def MarketName(self, value: str):
        self._MarketName = value

    @property
    def ExchangeID(self):
        return self._ExchangeID

    @ExchangeID.setter
    def ExchangeID(self, value: str):
        self._ExchangeID = value

    @property
    def InstrumentID(self):
        return self._InstrumentID

    @InstrumentID.setter
    def InstrumentID(self, value: str):
        self._InstrumentID = value

    @property
    def Range(self):
        return self._Range

    @Range.setter
    def Range(self, value: int):
        self._Range = value

    @property
    def TradingDay(self):
        return self._TradingDay

    @TradingDay.setter
    def TradingDay(self, value: str):
        self._TradingDay = value

    @property
    def ActionDayList(self):
        return self._ActionDayList

    @ActionDayList.setter
    def ActionDayList(self, value: List[str]):
        self._ActionDayList = value

    @property
    def PreSettlementPrice(self):
        return self._PreSettlementPrice

    @PreSettlementPrice.setter
    def PreSettlementPrice(self, value: float):
        self._PreSettlementPrice = value

    @property
    def ActionTimespanList(self):
        return self._ActionTimespanList

    @ActionTimespanList.setter
    def ActionTimespanList(self, value: List[int]):
        self._ActionTimespanList = value

    @property
    def TradingTimeList(self):
        return self._TradingTimeList

    @TradingTimeList.setter
    def TradingTimeList(self, value: List[str]):
        self._TradingTimeList = value

    @property
    def StartTimeList(self):
        return self._StartTimeList

    @StartTimeList.setter
    def StartTimeList(self, value: List[str]):
        self._StartTimeList = value

    @property
    def EndTimeList(self):
        return self._EndTimeList

    @EndTimeList.setter
    def EndTimeList(self, value: List[str]):
        self._EndTimeList = value

    @property
    def TotalTurnoverList(self):
        return self._TotalTurnoverList

    @TotalTurnoverList.setter
    def TotalTurnoverList(self, value: List[float]):
        self._TotalTurnoverList = value

    @property
    def OpenInterestList(self):
        return self._OpenInterestList

    @OpenInterestList.setter
    def OpenInterestList(self, value: List[float]):
        self._OpenInterestList = value

    @property
    def OpenPriceList(self):
        return self._OpenPriceList

    @OpenPriceList.setter
    def OpenPriceList(self, value: List[float]):
        self._OpenPriceList = value

    @property
    def OpenBidPriceList(self):
        return self._OpenBidPriceList

    @OpenBidPriceList.setter
    def OpenBidPriceList(self, value: List[float]):
        self._OpenBidPriceList = value

    @property
    def OpenAskPriceList(self):
        return self._OpenAskPriceList

    @OpenAskPriceList.setter
    def OpenAskPriceList(self, value: List[float]):
        self._OpenAskPriceList = value

    @property
    def OpenBidVolumeList(self):
        return self._OpenBidVolumeList

    @OpenBidVolumeList.setter
    def OpenBidVolumeList(self, value: List[int]):
        self._OpenBidVolumeList = value

    @property
    def OpenAskVolumeList(self):
        return self._OpenAskVolumeList

    @OpenAskVolumeList.setter
    def OpenAskVolumeList(self, value: List[int]):
        self._OpenAskVolumeList = value

    @property
    def HighPriceList(self):
        return self._HighPriceList

    @HighPriceList.setter
    def HighPriceList(self, value: List[float]):
        self._HighPriceList = value

    @property
    def HighBidPriceList(self):
        return self._HighBidPriceList

    @HighBidPriceList.setter
    def HighBidPriceList(self, value: List[float]):
        self._HighBidPriceList = value

    @property
    def HighAskPriceList(self):
        return self._HighAskPriceList

    @HighAskPriceList.setter
    def HighAskPriceList(self, value: List[float]):
        self._HighAskPriceList = value

    @property
    def HighBidVolumeList(self):
        return self._HighBidVolumeList

    @HighBidVolumeList.setter
    def HighBidVolumeList(self, value: List[int]):
        self._HighBidVolumeList = value

    @property
    def HighAskVolumeList(self):
        return self._HighAskVolumeList

    @HighAskVolumeList.setter
    def HighAskVolumeList(self, value: List[int]):
        self._HighAskVolumeList = value

    @property
    def LowerPriceList(self):
        return self._LowerPriceList

    @LowerPriceList.setter
    def LowerPriceList(self, value: List[float]):
        self._LowerPriceList = value

    @property
    def LowerBidPriceList(self):
        return self._LowerBidPriceList

    @LowerBidPriceList.setter
    def LowerBidPriceList(self, value: List[float]):
        self._LowerBidPriceList = value

    @property
    def LowerAskPriceList(self):
        return self._LowerAskPriceList

    @LowerAskPriceList.setter
    def LowerAskPriceList(self, value: List[float]):
        self._LowerAskPriceList = value

    @property
    def LowerBidVolumeList(self):
        return self._LowerBidVolumeList

    @LowerBidVolumeList.setter
    def LowerBidVolumeList(self, value: List[int]):
        self._LowerBidVolumeList = value

    @property
    def LowerAskVolumeList(self):
        return self._LowerAskVolumeList

    @LowerAskVolumeList.setter
    def LowerAskVolumeList(self, value: List[int]):
        self._LowerAskVolumeList = value

    @property
    def ClosePriceList(self):
        return self._ClosePriceList

    @ClosePriceList.setter
    def ClosePriceList(self, value: List[float]):
        self._ClosePriceList = value

    @property
    def CloseBidPriceList(self):
        return self._CloseBidPriceList

    @CloseBidPriceList.setter
    def CloseBidPriceList(self, value: List[float]):
        self._CloseBidPriceList = value

    @property
    def CloseAskPriceList(self):
        return self._CloseAskPriceList

    @CloseAskPriceList.setter
    def CloseAskPriceList(self, value: List[float]):
        self._CloseAskPriceList = value

    @property
    def CloseBidVolumeList(self):
        return self._CloseBidVolumeList

    @CloseBidVolumeList.setter
    def CloseBidVolumeList(self, value: List[int]):
        self._CloseBidVolumeList = value

    @property
    def CloseAskVolumeList(self):
        return self._CloseAskVolumeList

    @CloseAskVolumeList.setter
    def CloseAskVolumeList(self, value: List[int]):
        self._CloseAskVolumeList = value



from ...interface import IPacker


class MarkerGraphParamDataPacker(IPacker):
    def __init__(self, obj) -> None:
        super().__init__(obj)

    def obj_to_tuple(self):
        return [str(self._obj.ID), str(self._obj.Name), int(self._obj.PlotIndex), int(self._obj.ValueAxisID),
                str(self._obj.Text), str(self._obj.TextColor), int(self._obj.TextVAlign), int(self._obj.TextHAlign),
                int(self._obj.Orientation), int(self._obj.LineDirec), int(self._obj.LineWidth), int(self._obj.LineStyle),
                str(self._obj.LineColor), float(self._obj.Key), float(self._obj.Value), int(self._obj.MillTimeSpan),
                bool(self._obj.JoinValueAxis), bool(self._obj.ShowLegend), str(self._obj.LegendFormat),
                str(self._obj.LegendColor), dict(self._obj.UserData), bool(self._obj.Visible)]

    def tuple_to_obj(self, t):
        if len(t) >= 22:
            self._obj.ID = t[0]
            self._obj.Name = t[1]
            self._obj.PlotIndex = t[2]
            self._obj.ValueAxisID = t[3]
            self._obj.Text = t[4]
            self._obj.TextColor = t[5]
            self._obj.TextVAlign = t[6]
            self._obj.TextHAlign = t[7]
            self._obj.Orientation = t[8]
            self._obj.LineDirec = t[9]
            self._obj.LineWidth = t[10]
            self._obj.LineStyle = t[11]
            self._obj.LineColor = t[12]
            self._obj.Key = t[13]
            self._obj.Value = t[14]
            self._obj.MillTimeSpan = t[15]
            self._obj.JoinValueAxis = t[16]
            self._obj.ShowLegend = t[17]
            self._obj.LegendFormat = t[18]
            self._obj.LegendColor = t[19]
            self._obj.UserData = t[20]
            self._obj.Visible = t[21]

            return True
        return False



from typing import List, Dict, Tuple
from .client import FinClient
from .data.login_data import LoginData
from .data.market.ohlc_data import OHLCData
from .data.chart.chart_init_param_data import ChartInitParamData
from .data.chart.marker_graph_param_data import MarkerGraphParamData
from .data.chart.text_graph_param_data import TextGraphParamData
from .data.chart.financial_graph_param_data import FinancialGraphParamData
from .data.chart.line_graph_param_data import LineGraphParamData
from .data.chart.ohlc_value_data import OHLCValueData
from .data.chart.graph_value_data import GraphValueData
from .data.trade.order_data import OrderData
from .data.market.market_param_data import MarketParamData
from .data.market.query_param_data import QueryParamData
from .data.market.sub_ohlc_param_data import SubOHLCParamData
from .data.market.history_ohlc_param_data import HistoryOHLCParamData
from .data.market.save_ohlc_list_param_data import SaveOHLCListParamData
from .data.market.save_tick_list_param_data import SaveTickListParamData
from .data.trade.trade_param_data import TradeParamData
from .data.trade.read_history_order_param_data import ReadHistoryOrderParamData
from .data.trade.read_history_tradeorder_param_data import ReadHistoryTradeOrderParamData
from .data.trade.get_account_param_data import GetAccountParamData
from .data.trade.get_orders_param_data import GetOrdersParamData
from .data.trade.get_tradeorders_param_data import GetTradeOrdersParamData
from .data.trade.get_positions_param_data import GetPositionsParamData
from .data.chart.bar_graph_param_data import BarGraphParamData


def conncet(host: str = None, port: int = None, ):
    return FinClient.instance().connect(host, port)


def login(user_id: str = '', password: str = ''):
    return FinClient.instance().base_handle().login(LoginData(user_id, password))


def close():
    FinClient.instance().close()


def set_callback(**kwargs):
    '''
    设置行情回调
    :param kwargs OnTick=None,
    '''
    FinClient.instance().set_callback(**kwargs)


def set_auth_params(userid, password, host: str = None, port: int = None):
    '''
    设置登录信息
    :param userid:用户名
    :param password:密码
    :param host:网络地址默认为None
    :param port:端口号默认为None
    :return:result msg
    '''
    ret = conncet(host, port)
    if ret is None or ret[0] is False:
        return ret
    return login(userid, password)


def set_chart_init_params(params: ChartInitParamData):
    return FinClient.instance().chart_handle().set_chart_init_params(params)


def add_line_graph(id: str, plot_index=0, value_axis_id=-1, color: str = '#FFF', style=0, price_tick=0.01, tick_valid_mul=-1.0, bind_ins_id='', bind_range=''):
    '''
    添加线图
    :param id:图形ID
    :param plot_index:所在图层索引
    :param value_axis_id:所属Y轴
    :param color:颜色
    :param style:样式
    :param price_tick:最小变动刻度
    :param tick_valid_mul:显示有效的倍数 -1.0不做限制
    :param bind_ins_id:绑定合约
    :param bind_range:绑定合约周期
    :return:result msg
    '''
    return FinClient.instance().chart_handle().add_line_graph(
        LineGraphParamData(
            name=id,
            id=id,
            plot_index=plot_index,
            value_axis_id=value_axis_id,
            style=style,
            color=color,
            price_tick=price_tick,
            tick_valid_mul=tick_valid_mul,
            bind_ins_id=bind_ins_id,
            bind_range=bind_range)
    )


def add_bar_graph(id: str, plot_index=0, value_axis_id=-1, color: str = '#FFF', style=0, frame_style=2):
    """添加柱状图

    Args:
        id (str): 图形id
        plot_index (int, optional): 所在图层索引. Defaults to 0.
        value_axis_id (int, optional): 所属Y轴. Defaults to -1左边第一个Y轴.
        color (str, optional): 颜色. Defaults to '#FFF'.
        style (int, optional): 样式. Defaults to 0 box.
        frame_style (int, optional): 边框样式. Defaults to 2 线型.

    Returns:
        _type_: [bool,str]
    """

    return FinClient.instance().chart_handle().add_bar_graph(
        BarGraphParamData(
            name=id,
            id=id,
            plot_index=plot_index,
            valueaxis_id=value_axis_id,
            style=style,
            frame_style=frame_style,
            color=color,
        )
    )


def add_financial_graph(id: str, plot_index=0, value_axis_id=-1, style=0, price_tick=0.01, tick_valid_mul=-1.0, bind_ins_id='', bind_range=''):
    '''
    添加线图
    :param id:图形编号
    :param name:图形名称
    :param style:样式
    :param plot_index:所在图层索引
    :param value_axis_id:所属Y轴
    :param price_tick:最小变动刻度
    :param tick_valid_mul:显示有效的倍数 -1.0不做限制
    :param bind_ins_id:绑定合约
    :param bind_range:绑定合约周期
    :return: [result,msg] True 添加成功, False 添加失败
    '''
    return FinClient.instance().chart_handle().add_financial_graph(
        FinancialGraphParamData(
            id=id,
            name=id,
            style=style,
            plot_index=plot_index,
            value_axis_id=value_axis_id,
            price_tick=price_tick,
            tick_valid_mul=tick_valid_mul,
            bind_ins_id=bind_ins_id,
            bind_range=bind_range)
    )


def chart_init_show():
    return FinClient.instance().chart_handle().chart_init_show()


def add_line_value(graphid: str, key: float = 0.0, value: float = 0.0, mill_ts: int = -1):
    return FinClient.instance().chart_handle().add_graph_value(GraphValueData(
        graph_id=graphid,
        key=key,
        mill_ts=mill_ts,
        value=value)
    )


def add_marker_graph(param: MarkerGraphParamData):
    return FinClient.instance().chart_handle().add_marker_graph(param)


def add_graph_value(gv: GraphValueData):
    return FinClient.instance().chart_handle().add_graph_value(gv)


def add_graph_value_list(gvl):
    gvdl = []
    for gv in gvl:
        gvdl.append(GraphValueData(graph_id=gv[0], mill_ts=gv[1], value=gv[2]))
    return FinClient.instance().chart_handle().add_graph_value_list(gvdl)


def add_timespan_graphvalue_list(timespans: List[int], graph_values: Dict[str, List[float]] = {}, ohlc_values: Dict[str, Tuple[List[float], List[float], List[float], List[float]]] = {}):
    return FinClient.instance().chart_handle().add_timespan_graphvalue_list(timespans, graph_values, ohlc_values)


def add_ohlc_value(ov: OHLCValueData):
    return FinClient.instance().chart_handle().add_ohlc_value(ov)


def add_ohlc_value_list(ovl: List[OHLCValueData]):
    return FinClient.instance().chart_handle().add_ohlc_value_list(ovl)


def add_ohlc(graph_id: str, o: OHLCData):
    '''
    添加OHLC值
    :param graph_id:图形名称
    :param o:ohlc
    :return:result,msg
    '''
    return FinClient.instance().chart_handle().add_ohlc_value(
        OHLCValueData(
            graph_id=graph_id,
            ohlc_data=o)
    )


def draw_text(plot_index: int, value_axis_id: int, key: float, value: float, text: str, color: str = '#FFF'):
    '''
    画文本
    :param plot_index:所在图层索引
    :param value_axis_id:所属Y轴
    :param key:x轴值
    :param value:y轴值
    :param text:文本
    :param color:颜色
    :return:[result,msg]
    '''
    return FinClient.instance().chart_handle().add_text_graph(
        TextGraphParamData(
            plot_index=plot_index,
            value_axis_id=value_axis_id,
            key=key,
            value=value,
            text=text,
            color=color)
    )


def add_text_graph(param: TextGraphParamData):
    return FinClient.instance().chart_handle().add_text_graph(param)


def draw_text_milltime(plot_index, value_axis_id, mill_ts, value, text, color='#FFF'):
    '''
    画文本
    :param plot_index:所在图层索引
    :param value_axis_id:所属Y轴
    :param mill_ts:x时间戳
    :param value:y轴值
    :param text:文本
    :param color:颜色
    :return:result,msg
    '''
    return FinClient.instance().chart_handle().add_text_graph(
        TextGraphParamData(
            plot_index=plot_index,
            value_axis_id=value_axis_id,
            mill_ts=mill_ts,
            value=value,
            text=text,
            color=color)
    )


def set_market_params(market_names):
    '''
    设置行情参数
    :param market_names:行情名称多个时候用逗号分隔
    :return:result,msg
    '''
    return FinClient.instance().market_handle().set_market_params(
        MarketParamData(market_names=market_names)
    )


def subscribe(market_name: str, exchang_id: str, instrument_id: str):
    '''
    订阅行情
    :param market_name:行情名称
    :param exchang_id:交易所编码
    :param instrument_id:合约编码
    :return:result,msg
    '''
    return FinClient.instance().market_handle().subscribe(
        QueryParamData(
            market_name=market_name,
            exchange_id=exchang_id,
            instrument_id=instrument_id)
    )


def subscribe_ohlc(market_name: str, exchang_id: str, instrument_id: str, range: str):
    '''
    订阅行情
    :param market_name:行情名称
    :param exchang_id:交易所编码
    :param instrument_id:合约编码
    :param range:周期
    :return:result,msg
    '''
    return FinClient.instance().market_handle().subscribe_ohlc(
        SubOHLCParamData(
            market_name=market_name,
            exchange_id=exchang_id,
            instrument_id=instrument_id,
            range=range)
    )


def get_history_ohlc(market_name: str, exchang_id: str, instrument_id: str, range: str,
                     start_date: str, end_date: str, is_return_list: bool = False):
    '''
    获取历史ohlc数据
    :param market_name:行情名称
    :param exchang_id:交易所编码
    :param instrument_id:合约编码
    :param range:周期
    :param start_date 开始日期
    :param end_date 结束日期
    :param is_return_list 是否返回list格式
    :return:result,msg
    '''
    return FinClient.instance().market_handle().get_history_ohlc(
        HistoryOHLCParamData(
            market_name=market_name,
            exchange_id=exchang_id,
            instrument_id=instrument_id,
            range=range,
            start_date=start_date,
            end_date=end_date,
            is_return_list=is_return_list)
    )


def save_ohlc_list(
    market_name: str = '',
    exchange_id: str = '',
    instrument_id: str = '',
    range: int = 60,
    trading_day: str = '',
    pre_settlement_price: float = 0.0,
    action_day_list: List[str] = [],
    action_timespan_list: List[int] = [],
    trading_time_list: List[str] = [],
    start_time_list: List[str] = [],
    end_time_list: List[str] = [],
    total_turnover_list: List[float] = [],
    open_interest_list: List[float] = [],
    open_price_list: List[float] = [],
    open_bid_price_list: List[float] = [],
    open_ask_price_list: List[float] = [],
    open_bid_volume_list: List[int] = [],
    open_ask_volume_list: List[int] = [],
    high_price_list: List[float] = [],
    high_bid_price_list: List[float] = [],
    high_ask_price_list: List[float] = [],
    high_bid_volume_list: List[int] = [],
    high_ask_volume_list: List[int] = [],
    lower_price_list: List[float] = [],
    lower_bid_price_list: List[float] = [],
    lower_ask_price_list: List[float] = [],
    lower_bid_volume_list: List[int] = [],
    lower_ask_volume_list: List[int] = [],
    close_price_list: List[float] = [],
    close_bid_price_list: List[float] = [],
    close_ask_price_list: List[float] = [],
    close_bid_volume_list: List[int] = [],
    close_ask_volume_list: List[int] = []
):
    """批量按天保存OHLC数据

    Args:
        market_name (str, optional): 行情名称. Defaults to ''.
        exchange_id (str, optional): 交易所编码. Defaults to ''.
        instrument_id (str, optional): 合约编码. Defaults to ''.
        range (int, optional): 周期. Defaults to 60.
        trading_day (str, optional): 交易日. Defaults to ''.
        pre_settlement_price (float, optional): 昨结算价. Defaults to 0.0.
        action_day_list (List[str], optional): 自然日. Defaults to ''.
        action_timespan_list (List[int], optional): 自然日时间戳. Defaults to [].
        trading_time_list (List[str], optional): 交易时间. Defaults to [].
        start_time_list (List[str], optional): 开始时间. Defaults to [].
        end_time_list (List[str], optional): 结束时间. Defaults to [].
        total_turnover_list (List[float], optional): 成交金额. Defaults to [].
        open_interest_list (List[float], optional): 持仓量. Defaults to [].
        open_price_list (List[float], optional): 开盘价. Defaults to [].
        open_bid_price_list (List[float], optional): 开买价. Defaults to [].
        open_ask_price_list (List[float], optional): 开卖价. Defaults to [].
        open_bid_volume_list (List[int], optional): 开买量. Defaults to [].
        open_ask_volume_list (List[int], optional): 开卖量. Defaults to [].
        high_price_list (List[float], optional): 最高价. Defaults to [].
        high_bid_price_list (List[float], optional): 高买价. Defaults to [].
        high_ask_price_list (List[float], optional): 高卖价. Defaults to [].
        high_bid_volume_list (List[int], optional): 高买量. Defaults to [].
        high_ask_volume_list (List[int], optional): 高卖量. Defaults to [].
        lower_price_list (List[float], optional): 最低价. Defaults to [].
        lower_bid_price_list (List[float], optional): 低买价. Defaults to [].
        lower_ask_price_list (List[float], optional): 低卖价. Defaults to [].
        lower_bid_volume_list (List[int], optional): 低买量. Defaults to [].
        lower_ask_volume_list (List[int], optional): 低卖量. Defaults to [].
        close_price_list (List[float], optional): 收价. Defaults to [].
        close_bid_price_list (List[float], optional): 收买价. Defaults to [].
        close_ask_price_list (List[float], optional): 收卖价. Defaults to [].
        close_bid_volume_list (List[int], optional): 收买量. Defaults to [].
        close_ask_volume_list (List[int], optional): 收卖量. Defaults to [].

    Returns:
        _type_: [bool,str]
    """
    return FinClient.instance().market_handle().save_ohlc_list(
        SaveOHLCListParamData(market_name=market_name,
                              exchange_id=exchange_id,
                              instrument_id=instrument_id,
                              range=range,
                              trading_day=trading_day,
                              action_day_list=action_day_list,
                              pre_settlement_price=pre_settlement_price,
                              action_timespan_list=action_timespan_list,
                              trading_time_list=trading_time_list,
                              start_time_list=start_time_list,
                              end_time_list=end_time_list,
                              total_turnover_list=total_turnover_list,
                              open_interest_list=open_interest_list,
                              open_price_list=open_price_list,
                              open_bid_price_list=open_bid_price_list,
                              open_ask_price_list=open_ask_price_list,
                              open_bid_volume_list=open_bid_volume_list,
                              open_ask_volume_list=open_ask_volume_list,
                              high_price_list=high_price_list,
                              high_bid_price_list=high_bid_price_list,
                              high_ask_price_list=high_ask_price_list,
                              high_bid_volume_list=high_bid_volume_list,
                              high_ask_volume_list=high_ask_volume_list,
                              lower_price_list=lower_price_list,
                              lower_bid_price_list=lower_bid_price_list,
                              lower_ask_price_list=lower_ask_price_list,
                              lower_bid_volume_list=lower_bid_volume_list,
                              lower_ask_volume_list=lower_ask_volume_list,
                              close_price_list=close_price_list,
                              close_bid_price_list=close_bid_price_list,
                              close_ask_price_list=close_ask_price_list,
                              close_bid_volume_list=close_bid_volume_list,
                              close_ask_volume_list=close_ask_volume_list)
    )


def save_tick_list(
    market_name: str = '',
    exchange_id: str = '',
    instrument_id: str = '',
    product_id: str = '',
    trading_day: str = '',
    pre_close_price: float = float("NaN"),
    pre_settlement_price: float = float("NaN"),
    pre_open_interest: float = float("NaN"),
    upper_limit_price: float = float("NaN"),
    lower_limit_price: float = float("NaN"),
    action_day_list: List[str] = [],
    trading_time_list: List[str] = [],
    update_mill_sec_list: List[int] = [],
    local_time_list: List[int] = [],
    last_price_list: List[float] = [],
    last_volume_list: List[int] = [],
    bid_price_list: List[float] = [],
    bid_volume_list: List[int] = [],
    ask_price_list: List[float] = [],
    ask_volume_list: List[int] = [],
    avg_price_list: List[float] = [],
    open_price_list: List[float] = [],
    high_price_list: List[float] = [],
    lower_price_list: List[float] = [],
    total_turnover_list: List[float] = [],
    total_volume_list: List[int] = [],
    open_interest_list: List[float] = [],
    close_price_list: List[float] = [],
    settlement_price_list: List[float] = [],
    total_value_list: List[float] = []
):
    """批量按天保存行情数据

    Args:
        market_name (str, optional): 行情名称. Defaults to ''.
        exchange_id (str, optional): 交易所编码. Defaults to ''.
        instrument_id (str, optional): 合约编码. Defaults to ''.
        product_id (str, optional): 品种编码. Defaults to ''.
        trading_day (str, optional): 交易日. Defaults to ''.
        pre_close_price (float, optional): 昨收价. Defaults to float("NaN").
        pre_settlement_price (float, optional): 昨结算价. Defaults to float("NaN").
        pre_open_interest (float, optional): 昨持仓量. Defaults to float("NaN").
        upper_limit_price (float, optional): 涨停板价. Defaults to float("NaN").
        lower_limit_price (float, optional): 跌停板价. Defaults to float("NaN").
        action_day_list (List[str], optional): 自然日列表. Defaults to [].
        trading_time_list (List[str], optional): 行情时间. Defaults to [].
        update_mill_sec_list (List[int], optional): 最后修改毫秒值. Defaults to [].
        local_time_list (List[int], optional): 本地日期纳秒时间. Defaults to [].
        last_price_list (List[float], optional): 最新价. Defaults to [].
        last_volume_list (List[int], optional): 最新量. Defaults to [].
        bid_price_list (List[float], optional): 买价. Defaults to [].
        bid_volume_list (List[int], optional): 买量. Defaults to [].
        ask_price_list (List[float], optional): 卖价. Defaults to [].
        ask_volume_list (List[int], optional): 卖量. Defaults to [].
        avg_price_list (List[float], optional): 当日均价. Defaults to [].
        open_price_list (List[float], optional): 今开盘. Defaults to [].
        high_price_list (List[float], optional): 最高价. Defaults to [].
        lower_price_list (List[float], optional): 最低价. Defaults to [].
        total_turnover_list (List[float], optional): 成交金额. Defaults to [].
        total_volume_list (List[int], optional): 成交量. Defaults to [].
        open_interest_list (List[float], optional): 持仓量. Defaults to [].
        close_price_list (List[float], optional): 今收盘. Defaults to [].
        settlement_price_list (List[float], optional): 本次结算价. Defaults to [].
        total_value_list (List[float], optional): 总市值. Defaults to [].

    Returns:
        _type_: [bool,str]
    """
    return FinClient.instance().market_handle().save_tick_list(
        SaveTickListParamData(
            market_name=market_name,
            exchange_id=exchange_id,
            instrument_id=instrument_id,
            product_id=product_id,
            trading_day=trading_day,
            pre_close_price=pre_close_price,
            pre_settlement_price=pre_settlement_price,
            pre_open_interest=pre_open_interest,
            upper_limit_price=upper_limit_price,
            lower_limit_price=lower_limit_price,
            action_day_list=action_day_list,
            trading_time_list=trading_time_list,
            update_mill_sec_list=update_mill_sec_list,
            local_time_list=local_time_list,
            last_price_list=last_price_list,
            last_volume_list=last_volume_list,
            bid_price_list=bid_price_list,
            bid_volume_list=bid_volume_list,
            ask_price_list=ask_price_list,
            ask_volume_list=ask_volume_list,
            avg_price_list=avg_price_list,
            open_price_list=open_price_list,
            high_price_list=high_price_list,
            lower_price_list=lower_price_list,
            total_turnover_list=total_turnover_list,
            total_volume_list=total_volume_list,
            open_interest_list=open_interest_list,
            close_price_list=close_price_list,
            settlement_price_list=settlement_price_list,
            total_value_list=total_value_list
        )
    )


def set_trade_params(tradenames: str):
    return FinClient.instance().trade_handle().set_trade_params(
        TradeParamData(
            trade_names=tradenames)
    )


def insert_order(trade_name, exchange_id: str, instrument_id: str, direc: int, price: float, vol: int, open_close_type: int):
    return FinClient.instance().trade_handle().insert_order(
        OrderData(
            exchange_id=exchange_id,
            instrument_id=instrument_id,
            price=price,
            direction=direc,
            volume=vol,
            investor_id=trade_name,
            open_close_type=open_close_type
        )
    )


def cancel_order_by_data(order: OrderData):
    return FinClient.instance().trade_handle().cancel_order(order)


def cancel_order(trade_name: str, order_id: str, instrument_id: str, order_ref: str, price: float):
    return FinClient.instance().trade_handle().cancel_order(
        OrderData(
            investor_id=trade_name,
            order_id=order_id,
            instrument_id=instrument_id,
            order_ref=order_ref,
            price=price
        )
    )


def read_history_orders(start_date: str, end_date: str):
    return FinClient.instance().trade_handle().read_history_orders(
        ReadHistoryOrderParamData(
            start_date=start_date,
            end_date=end_date
        )
    )


def read_history_tradeorders(start_date: str, end_date: str):
    return FinClient.instance().trade_handle().read_history_tradeorders(
        ReadHistoryTradeOrderParamData(
            start_date=start_date,
            end_date=end_date
        )
    )


def get_orders(trade_name: str):
    return FinClient.instance().trade_handle().get_orders(
        GetOrdersParamData(
            trade_name=trade_name
        )
    )


def get_tradeorders(trade_name: str):
    return FinClient.instance().trade_handle().get_tradeorders(
        GetTradeOrdersParamData(
            trade_name=trade_name
        )
    )


def get_positions(trade_name: str):
    return FinClient.instance().trade_handle().get_positions(
        GetPositionsParamData(
            trade_name=trade_name
        )
    )


def get_account(trade_name: str):
    return FinClient.instance().trade_handle().get_account(
        GetAccountParamData(
            trade_name=trade_name
        )
    )


def save_chart_data(file_name: str):
    '''
    保存图数据
    :param file_name 文件名称
    :return [result,msg]
    '''
    return FinClient.instance().chart_handle().save_chart_data(file_name)


def load_chart_data(file_name: str):
    '''
    加载图数据
    :param file_name 文件名称
    :return [result,msg]
    '''
    return FinClient.instance().chart_handle().load_chart_data(file_name)



from ...interface import IPacker


class HistoryOHLCParamDataPacker(IPacker):
    def __init__(self, obj) -> None:
        super().__init__(obj)

    def obj_to_tuple(self):
        return [str(self._obj.MarketName), str(self._obj.ExchangeID), str(self._obj.InstrumentID), str(self._obj.Range),
                str(self._obj.StartDate), str(self._obj.EndDate), list(self._obj.OHLCList)]

    def tuple_to_obj(self, t):
        if len(t) >= 7:
            self._obj.MarketName = t[0]
            self._obj.ExchangeID = t[1]
            self._obj.InstrumentID = t[2]
            self._obj.Range = t[3]
            self._obj.StartDate = t[4]
            self._obj.EndDate = t[5]
            self._obj.OHLCList = t[6]

            return True
        return False



from ...interface import IData
from ...packer.trade.position_data_packer import PositionDataPacker


class PositionData(IData):
    def __init__(self, investor_id: str = '', broker_id: str = '', exchange_id: str = '', product_id: str = '', instrument_id: str = '',
                 instrument_name: str = '', delivery_month: str = '', position_buy_today: int = 0, position_buy_yesterday: int = 0, position_buy: int = 0,
                 position_sell_today: int = 0, position_sell_yesterday: int = 0, position_sell: int = 0, position_total: int = 0, cancele_dorder_count: int = 0,
                 add_order_count: int = 0, sum_trade_volume: int = 0, self_trade_count: int = 0, error_order_count: int = 0,
                 buy_open_sum: int = 0, sell_open_sum: int = 0, buy_sum: int = 0, sell_sum: int = 0, buy_sum_price: float = 0, sell_sum_price: float = 0,
                 untrade_buy: int = 0, untrade_sell: int = 0, untrade_open: int = 0, untrade_buy_open: int = 0, untrade_sell_open: int = 0,
                 untrade_close: int = 0, untrade_buy_close: int = 0, untrade_sell_close: int = 0, buy_ydposition: int = 0, sell_ydposition: int = 0,
                 buy_pre_settlement_price: float = 0.0, sell_pre_settlement_price: float = 0.0, long_avg_price: float = 0.0, short_avg_price: float = 0.0,
                 frozen_margin: float = 0.0, long_frozen_margin: float = 0.0, short_frozen_margin: float = 0.0, frozen_commission: float = 0.0,
                 open_frozen_commission: float = 0.0, close_frozen_commission: float = 0.0, close_today_frozen_commission: float = 0.0,
                 close_profit: float = 0.0, long_close_profit: float = 0.0, short_close_profit: float = 0.0, curr_margin: float = 0.0,
                 long_curr_margin: float = 0.0, short_curr_margin: float = 0.0, short_base_margin: float = 0.0, short_pos_margin: float = 0.0,
                 commission: float = 0.0, open_commission: float = 0.0, close_commission: float = 0.0, close_today_commission: float = 0.0,
                 position_profit: float = 0.0, long_position_profit: float = 0.0, short_position_profit: float = 0.0, order_commission: float = 0.0,
                 royalty_position_profit: float = 0.0, long_royalty_position_profit: float = 0.0, short_royalty_position_profit: float = 0.0,
                 lock_buy_open: int = 0, lock_buy_close: int = 0, lock_buy_close_today: int = 0,
                 lock_sell_open: int = 0, lock_sell_close: int = 0, lock_sell_close_today: int = 0):
        super().__init__(PositionDataPacker(self))
        self._InvestorID: str = investor_id
        self._BrokerID: str = broker_id
        self._ExchangeID: str = exchange_id
        self._ProductID: str = product_id
        self._InstrumentID: str = instrument_id
        self._InstrumentName: str = instrument_name
        self._DeliveryMonth: str = delivery_month
        self._PositionBuyToday: int = position_buy_today
        self._PositionBuyYesterday: int = position_buy_yesterday
        self._PositionBuy: int = position_buy
        self._PositionSellToday: int = position_sell_today
        self._PositionSellYesterday: int = position_sell_yesterday
        self._PositionSell: int = position_sell
        self._PositionTotal: int = position_total
        self._CanceledOrderCount: int = cancele_dorder_count
        self._AddOrderCount: int = add_order_count
        self._SumTradeVolume: int = sum_trade_volume
        self._SelfTradeCount: int = self_trade_count
        self._ErrorOrderCount: int = error_order_count
        self._BuyOpenSum: int = buy_open_sum
        self._SellOpenSum: int = sell_open_sum
        self._BuySum: int = buy_sum
        self._SellSum: int = sell_sum
        self._BuySumPrice: float = buy_sum_price
        self._SellSumPrice: float = sell_sum_price
        self._UnTradeBuy: int = untrade_buy
        self._UnTradeSell: int = untrade_sell
        self._UntradeOpen: int = untrade_open
        self._UntradeBuyOpen: int = untrade_buy_open
        self._UntradeSellOpen: int = untrade_sell_open
        self._UntradeClose: int = untrade_close
        self._UntradeBuyClose: int = untrade_buy_close
        self._UntradeSellClose: int = untrade_sell_close
        self._BuyYdPosition: int = buy_ydposition
        self._SellYdPosition: int = sell_ydposition
        self._BuyPreSettlementPrice: float = buy_pre_settlement_price
        self._SellPreSettlementPrice: float = sell_pre_settlement_price
        self._LongAvgPrice: float = long_avg_price
        self._ShortAvgPrice: float = short_avg_price
        self._FrozenMargin: float = frozen_margin
        self._LongFrozenMargin: float = long_frozen_margin
        self._ShortFrozenMargin: float = short_frozen_margin
        self._FrozenCommission: float = frozen_commission
        self._OpenFrozenCommission: float = open_frozen_commission
        self._CloseFrozenCommission: float = close_frozen_commission
        self._CloseTodayFrozenCommission: float = close_today_frozen_commission
        self._CloseProfit: float = close_profit
        self._LongCloseProfit: float = long_close_profit
        self._ShortCloseProfit: float = short_close_profit
        self._CurrMargin: float = curr_margin
        self._LongCurrMargin: float = long_curr_margin
        self._ShortCurrMargin: float = short_curr_margin
        self._ShortBaseMargin: float = short_base_margin
        self._ShortPosMargin: float = short_pos_margin
        self._Commission: float = commission
        self._OpenCommission: float = open_commission
        self._CloseCommission: float = close_commission
        self._CloseTodayCommission: float = close_today_commission
        self._PositionProfit: float = position_profit
        self._LongPositionProfit: float = long_position_profit
        self._ShortPositionProfit: float = short_position_profit
        self._OrderCommission: float = order_commission
        self._RoyaltyPositionProfit: float = royalty_position_profit
        self._LongRoyaltyPositionProfit: float = long_royalty_position_profit
        self._ShortRoyaltyPositionProfit: float = short_royalty_position_profit
        self._LockBuyOpen: int = lock_buy_open
        self._LockBuyClose: int = lock_buy_close
        self._LockBuyCloseToday: int = lock_buy_close_today
        self._LockSellOpen: int = lock_sell_open
        self._LockSellClose: int = lock_sell_close
        self._LockSellCloseToday: int = lock_sell_close_today

    @property
    def InvestorID(self):
        return self._InvestorID

    @InvestorID.setter
    def InvestorID(self, value: str):
        self._InvestorID = value

    @property
    def BrokerID(self):
        return self._BrokerID

    @BrokerID.setter
    def BrokerID(self, value: str):
        self._BrokerID = value

    @property
    def ExchangeID(self):
        return self._ExchangeID

    @ExchangeID.setter
    def ExchangeID(self, value: str):
        self._ExchangeID = value

    @property
    def ProductID(self):
        return self._ProductID

    @ProductID.setter
    def ProductID(self, value: str):
        self._ProductID = value

    @property
    def InstrumentID(self):
        return self._InstrumentID

    @InstrumentID.setter
    def InstrumentID(self, value: str):
        self._InstrumentID = value

    @property
    def InstrumentName(self):
        return self._InstrumentName

    @InstrumentName.setter
    def InstrumentName(self, value: str):
        self._InstrumentName = value

    @property
    def DeliveryMonth(self):
        return self._DeliveryMonth

    @DeliveryMonth.setter
    def DeliveryMonth(self, value: str):
        self._DeliveryMonth = value

    @property
    def PositionBuyToday(self):
        return self._PositionBuyToday

    @PositionBuyToday.setter
    def PositionBuyToday(self, value: int):
        self._PositionBuyToday = value

    @property
    def PositionBuyYesterday(self):
        return self._PositionBuyYesterday

    @PositionBuyYesterday.setter
    def PositionBuyYesterday(self, value: int):
        self._PositionBuyYesterday = value

    @property
    def PositionBuy(self):
        return self._PositionBuy

    @PositionBuy.setter
    def PositionBuy(self, value: int):
        self._PositionBuy = value

    @property
    def PositionSellToday(self):
        return self._PositionSellToday

    @PositionSellToday.setter
    def PositionSellToday(self, value: int):
        self._PositionSellToday = value

    @property
    def PositionSellYesterday(self):
        return self._PositionSellYesterday

    @PositionSellYesterday.setter
    def PositionSellYesterday(self, value: int):
        self._PositionSellYesterday = value

    @property
    def PositionSell(self):
        return self._PositionSell

    @PositionSell.setter
    def PositionSell(self, value: int):
        self._PositionSell = value

    @property
    def PositionTotal(self):
        return self._PositionTotal

    @PositionTotal.setter
    def PositionTotal(self, value: int):
        self._PositionTotal = value

    @property
    def CanceledOrderCount(self):
        return self._CanceledOrderCount

    @CanceledOrderCount.setter
    def CanceledOrderCount(self, value: int):
        self._CanceledOrderCount = value

    @property
    def AddOrderCount(self):
        return self._AddOrderCount

    @AddOrderCount.setter
    def AddOrderCount(self, value: int):
        self._AddOrderCount = value

    @property
    def SumTradeVolume(self):
        return self._SumTradeVolume

    @SumTradeVolume.setter
    def SumTradeVolume(self, value: int):
        self._SumTradeVolume = value

    @property
    def SelfTradeCount(self):
        return self._SelfTradeCount

    @SelfTradeCount.setter
    def SelfTradeCount(self, value: int):
        self._SelfTradeCount = value

    @property
    def ErrorOrderCount(self):
        return self._ErrorOrderCount

    @ErrorOrderCount.setter
    def ErrorOrderCount(self, value: int):
        self._ErrorOrderCount = value

    @property
    def BuyOpenSum(self):
        return self._BuyOpenSum

    @BuyOpenSum.setter
    def BuyOpenSum(self, value: int):
        self._BuyOpenSum = value

    @property
    def SellOpenSum(self):
        return self._SellOpenSum

    @SellOpenSum.setter
    def SellOpenSum(self, value: int):
        self._SellOpenSum = value

    @property
    def BuySum(self):
        return self._BuySum

    @BuySum.setter
    def BuySum(self, value: int):
        self._BuySum = value

    @property
    def SellSum(self):
        return self._SellSum

    @SellSum.setter
    def SellSum(self, value: int):
        self._SellSum = value

    @property
    def BuySumPrice(self):
        return self._BuySumPrice

    @BuySumPrice.setter
    def BuySumPrice(self, value: float):
        self._BuySumPrice = value

    @property
    def SellSumPrice(self):
        return self._SellSumPrice

    @SellSumPrice.setter
    def SellSumPrice(self, value: float):
        self._SellSumPrice = value

    @property
    def UnTradeBuy(self):
        return self._UnTradeBuy

    @UnTradeBuy.setter
    def UnTradeBuy(self, value: int):
        self._UnTradeBuy = value

    @property
    def UnTradeSell(self):
        return self._UnTradeSell

    @UnTradeSell.setter
    def UnTradeSell(self, value: int):
        self._UnTradeSell = value

    @property
    def UntradeOpen(self):
        return self._UntradeOpen

    @UntradeOpen.setter
    def UntradeOpen(self, value: int):
        self._UntradeOpen = value

    @property
    def UntradeBuyOpen(self):
        return self._UntradeBuyOpen

    @UntradeBuyOpen.setter
    def UntradeBuyOpen(self, value: int):
        self._UntradeBuyOpen = value

    @property
    def UntradeSellOpen(self):
        return self._UntradeSellOpen

    @UntradeSellOpen.setter
    def UntradeSellOpen(self, value: int):
        self._UntradeSellOpen = value

    @property
    def UntradeClose(self):
        return self._UntradeClose

    @UntradeClose.setter
    def UntradeClose(self, value: int):
        self._UntradeClose = value

    @property
    def UntradeBuyClose(self):
        return self._UntradeBuyClose

    @UntradeBuyClose.setter
    def UntradeBuyClose(self, value: int):
        self._UntradeBuyClose = value

    @property
    def UntradeSellClose(self):
        return self._UntradeSellClose

    @UntradeSellClose.setter
    def UntradeSellClose(self, value: int):
        self._UntradeSellClose = value

    @property
    def BuyYdPosition(self):
        return self._BuyYdPosition

    @BuyYdPosition.setter
    def BuyYdPosition(self, value: int):
        self._BuyYdPosition = value

    @property
    def SellYdPosition(self):
        return self._SellYdPosition

    @SellYdPosition.setter
    def SellYdPosition(self, value: int):
        self._SellYdPosition = value

    @property
    def BuyPreSettlementPrice(self):
        return self._BuyPreSettlementPrice

    @BuyPreSettlementPrice.setter
    def BuyPreSettlementPrice(self, value: float):
        self._BuyPreSettlementPrice = value

    @property
    def SellPreSettlementPrice(self):
        return self._SellPreSettlementPrice

    @SellPreSettlementPrice.setter
    def SellPreSettlementPrice(self, value: float):
        self._SellPreSettlementPrice = value

    @property
    def LongAvgPrice(self):
        return self._LongAvgPrice

    @LongAvgPrice.setter
    def LongAvgPrice(self, value: float):
        self._LongAvgPrice = value

    @property
    def ShortAvgPrice(self):
        return self._ShortAvgPrice

    @ShortAvgPrice.setter
    def ShortAvgPrice(self, value: float):
        self._ShortAvgPrice = value

    @property
    def FrozenMargin(self):
        return self._FrozenMargin

    @FrozenMargin.setter
    def FrozenMargin(self, value: float):
        self._FrozenMargin = value

    @property
    def LongFrozenMargin(self):
        return self._LongFrozenMargin

    @LongFrozenMargin.setter
    def LongFrozenMargin(self, value: float):
        self._LongFrozenMargin = value

    @property
    def ShortFrozenMargin(self):
        return self._ShortFrozenMargin

    @ShortFrozenMargin.setter
    def ShortFrozenMargin(self, value: float):
        self._ShortFrozenMargin = value

    @property
    def FrozenCommission(self):
        return self._FrozenCommission

    @FrozenCommission.setter
    def FrozenCommission(self, value: float):
        self._FrozenCommission = value

    @property
    def OpenFrozenCommission(self):
        return self._OpenFrozenCommission

    @OpenFrozenCommission.setter
    def OpenFrozenCommission(self, value: float):
        self._OpenFrozenCommission = value

    @property
    def CloseFrozenCommission(self):
        return self._CloseFrozenCommission

    @CloseFrozenCommission.setter
    def CloseFrozenCommission(self, value: float):
        self._CloseFrozenCommission = value

    @property
    def CloseTodayFrozenCommission(self):
        return self._CloseTodayFrozenCommission

    @CloseTodayFrozenCommission.setter
    def CloseTodayFrozenCommission(self, value: float):
        self._CloseTodayFrozenCommission = value

    @property
    def CloseProfit(self):
        return self._CloseProfit

    @CloseProfit.setter
    def CloseProfit(self, value: float):
        self._CloseProfit = value

    @property
    def LongCloseProfit(self):
        return self._LongCloseProfit

    @LongCloseProfit.setter
    def LongCloseProfit(self, value: float):
        self._LongCloseProfit = value

    @property
    def ShortCloseProfit(self):
        return self._ShortCloseProfit

    @ShortCloseProfit.setter
    def ShortCloseProfit(self, value: float):
        self._ShortCloseProfit = value

    @property
    def CurrMargin(self):
        return self._CurrMargin

    @CurrMargin.setter
    def CurrMargin(self, value: float):
        self._CurrMargin = value

    @property
    def LongCurrMargin(self):
        return self._LongCurrMargin

    @LongCurrMargin.setter
    def LongCurrMargin(self, value: float):
        self._LongCurrMargin = value

    @property
    def ShortCurrMargin(self):
        return self._ShortCurrMargin

    @ShortCurrMargin.setter
    def ShortCurrMargin(self, value: float):
        self._ShortCurrMargin = value

    @property
    def ShortBaseMargin(self):
        return self._ShortBaseMargin

    @ShortBaseMargin.setter
    def ShortBaseMargin(self, value: float):
        self._ShortBaseMargin = value

    @property
    def ShortPosMargin(self):
        return self._ShortPosMargin

    @ShortPosMargin.setter
    def ShortPosMargin(self, value: float):
        self._ShortPosMargin = value

    @property
    def Commission(self):
        return self._Commission

    @Commission.setter
    def Commission(self, value: float):
        self._Commission = value

    @property
    def OpenCommission(self):
        return self._OpenCommission

    @OpenCommission.setter
    def OpenCommission(self, value: float):
        self._OpenCommission = value

    @property
    def CloseCommission(self):
        return self._CloseCommission

    @CloseCommission.setter
    def CloseCommission(self, value: float):
        self._CloseCommission = value

    @property
    def CloseTodayCommission(self):
        return self._CloseTodayCommission

    @CloseTodayCommission.setter
    def CloseTodayCommission(self, value: float):
        self._CloseTodayCommission = value

    @property
    def PositionProfit(self):
        return self._PositionProfit

    @PositionProfit.setter
    def PositionProfit(self, value: float):
        self._PositionProfit = value

    @property
    def LongPositionProfit(self):
        return self._LongPositionProfit

    @LongPositionProfit.setter
    def LongPositionProfit(self, value: float):
        self._LongPositionProfit = value

    @property
    def ShortPositionProfit(self):
        return self._ShortPositionProfit

    @ShortPositionProfit.setter
    def ShortPositionProfit(self, value: float):
        self._ShortPositionProfit = value

    @property
    def OrderCommission(self):
        return self._OrderCommission

    @OrderCommission.setter
    def OrderCommission(self, value: float):
        self._OrderCommission = value

    @property
    def RoyaltyPositionProfit(self):
        return self._RoyaltyPositionProfit

    @RoyaltyPositionProfit.setter
    def RoyaltyPositionProfit(self, value: float):
        self._RoyaltyPositionProfit = value

    @property
    def LongRoyaltyPositionProfit(self):
        return self._LongRoyaltyPositionProfit

    @LongRoyaltyPositionProfit.setter
    def LongRoyaltyPositionProfit(self, value: float):
        self._LongRoyaltyPositionProfit = value

    @property
    def ShortRoyaltyPositionProfit(self):
        return self._ShortRoyaltyPositionProfit

    @ShortRoyaltyPositionProfit.setter
    def ShortRoyaltyPositionProfit(self, value: float):
        self._ShortRoyaltyPositionProfit = value

    @property
    def LockBuyOpen(self):
        return self._LockBuyOpen

    @LockBuyOpen.setter
    def LockBuyOpen(self, value: int):
        self._LockBuyOpen = value

    @property
    def LockBuyClose(self):
        return self._LockBuyClose

    @LockBuyClose.setter
    def LockBuyClose(self, value: int):
        self._LockBuyClose = value

    @property
    def LockBuyCloseToday(self):
        return self._LockBuyCloseToday

    @LockBuyCloseToday.setter
    def LockBuyCloseToday(self, value: int):
        self._LockBuyCloseToday = value

    @property
    def LockSellOpen(self):
        return self._LockSellOpen

    @LockSellOpen.setter
    def LockSellOpen(self, value: int):
        self._LockSellOpen = value

    @property
    def LockSellClose(self):
        return self._LockSellClose

    @LockSellClose.setter
    def LockSellClose(self, value: int):
        self._LockSellClose = value

    @property
    def LockSellCloseToday(self):
        return self._LockSellCloseToday

    @LockSellCloseToday.setter
    def LockSellCloseToday(self, value: int):
        self._LockSellCloseToday = value



from ...interface import IData
from ...packer.trade.tradeorder_data_packer import TradeOrderDataPacker


class TradeOrderData(IData):
    def __init__(self, exchange_id: str = '', product_id: str = '', instrument_id: str = '', trade_time: str = '',
                 trading_day: str = '', trade_date: str = '', broker_order_seq: str = '', order_id: str = '',
                 trade_id: str = '', price: float = 0.0, volume: int = 0, unclose_volume: int = 0, direction: int = 1,
                 open_close_type: int = 0, hedge_type: int = 0, is_yesterday_trade: bool = False,
                 close_profit: float = 0.0, curr_margin: float = 0.0, commission: float = 0.0, trade_type: int = 0,
                 rtn_tradeorder_local_time: int = 0):
        super().__init__(TradeOrderDataPacker(self))
        self._ExchangeID: str = exchange_id
        self._ProductID: str = product_id
        self._InstrumentID: str = instrument_id
        self._TradeTime: str = trade_time
        self._TradingDay: str = trading_day
        self._TradeDate: str = trade_date
        self._BrokerOrderSeq: str = broker_order_seq
        self._OrderID: str = order_id
        self._TradeID: str = trade_id
        self._Price: float = price
        self._Volume: int = volume
        self._UnCloseVolume: int = unclose_volume
        self._Direction: int = direction
        self._OpenCloseType: int = open_close_type
        self._HedgeType: int = hedge_type
        self._IsYesterdayTrade: bool = is_yesterday_trade
        self._CloseProfit: float = close_profit
        self._CurrMargin: float = curr_margin
        self._Commission: float = commission
        self._TradeType: int = trade_type
        self._RtnTradeOrderLocalTime: int = rtn_tradeorder_local_time

    @property
    def ExchangeID(self):
        return self._ExchangeID

    @ExchangeID.setter
    def ExchangeID(self, value: str):
        self._ExchangeID = value

    @property
    def ProductID(self):
        return self._ProductID

    @ProductID.setter
    def ProductID(self, value: str):
        self._ProductID = value

    @property
    def InstrumentID(self):
        return self._InstrumentID

    @InstrumentID.setter
    def InstrumentID(self, value: str):
        self._InstrumentID = value

    @property
    def TradeTime(self):
        return self._TradeTime

    @TradeTime.setter
    def TradeTime(self, value: str):
        self._TradeTime = value

    @property
    def TradingDay(self):
        return self._TradingDay

    @TradingDay.setter
    def TradingDay(self, value: str):
        self._TradingDay = value

    @property
    def TradeDate(self):
        return self._TradeDate

    @TradeDate.setter
    def TradeDate(self, value: str):
        self._TradeDate = value

    @property
    def BrokerOrderSeq(self):
        return self._BrokerOrderSeq

    @BrokerOrderSeq.setter
    def BrokerOrderSeq(self, value: str):
        self._BrokerOrderSeq = value

    @property
    def OrderID(self):
        return self._OrderID

    @OrderID.setter
    def OrderID(self, value: str):
        self._OrderID = value

    @property
    def TradeID(self):
        return self._TradeID

    @TradeID.setter
    def TradeID(self, value: str):
        self._TradeID = value

    @property
    def Price(self):
        return self._Price

    @Price.setter
    def Price(self, value: float):
        self._Price = value

    @property
    def Volume(self):
        return self._Volume

    @Volume.setter
    def Volume(self, value: int):
        self._Volume = value

    @property
    def UnCloseVolume(self):
        return self._UnCloseVolume

    @UnCloseVolume.setter
    def UnCloseVolume(self, value: int):
        self._UnCloseVolume = value

    @property
    def Direction(self):
        return self._Direction

    @Direction.setter
    def Direction(self, value: int):
        self._Direction = value

    @property
    def OpenCloseType(self):
        return self._OpenCloseType

    @OpenCloseType.setter
    def OpenCloseType(self, value: int):
        self._OpenCloseType = value

    @property
    def HedgeType(self):
        return self._HedgeType

    @HedgeType.setter
    def HedgeType(self, value: int):
        self._HedgeType = value

    @property
    def IsYesterdayTrade(self):
        return self._IsYesterdayTrade

    @IsYesterdayTrade.setter
    def IsYesterdayTrade(self, value: bool):
        self._IsYesterdayTrade = value

    @property
    def CloseProfit(self):
        return self._CloseProfit

    @CloseProfit.setter
    def CloseProfit(self, value: float):
        self._CloseProfit = value

    @property
    def CurrMargin(self):
        return self._CurrMargin

    @CurrMargin.setter
    def CurrMargin(self, value: float):
        self._CurrMargin = value

    @property
    def Commission(self):
        return self._Commission

    @Commission.setter
    def Commission(self, value: float):
        self._Commission = value

    @property
    def TradeType(self):
        return self._TradeType

    @TradeType.setter
    def TradeType(self, value: int):
        self._TradeType = value

    @property
    def RtnTradeOrderLocalTime(self):
        return self._RtnTradeOrderLocalTime

    @RtnTradeOrderLocalTime.setter
    def RtnTradeOrderLocalTime(self, value: int):
        self._RtnTradeOrderLocalTime = value



from ...interface import IPacker
from ...data.trade.tradeorder_data import TradeOrderData


class ReadHistoryTradeOrderParamDataPacker(IPacker):
    def __init__(self, obj) -> None:
        super().__init__(obj)

    def obj_to_tuple(self):
        order_data_list = []
        for od in self._obj.DataList:
            order_data_list.append(od.obj_to_tuple())
        return [str(self._obj.StartDate), str(self._obj.EndDate), order_data_list]

    def tuple_to_obj(self, t):
        if len(t) >= 3:
            self._obj.StartDate = t[0]
            self._obj.EndDate = t[1]
            for t in t[2]:
                od = TradeOrderData()
                od.tuple_to_obj(t)
                self._obj.DataList.append(od)

            return True
        return False



from setuptools import setup, find_packages

setup(
    name='rcquant_sdk',
    version='0.0.240108.15',
    description='rcquant_sdk',
    author='rcquant_sdk',
    author_email='rcquant_sdk@example.com',
    packages=find_packages(),
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    install_requires=[
        'msgpack',
        'numpy',
        'pandas',
    ],
    python_requires='>=3.6',
)



from ...interface import IPacker


class PositionDataPacker(IPacker):
    def __init__(self, obj) -> None:
        super().__init__(obj)

    def obj_to_tuple(self):
        return [str(self._obj.InvestorID), str(self._obj.BrokerID), str(self._obj.ExchangeID), str(self._obj.ProductID),
                str(self._obj.InstrumentID), str(self._obj.InstrumentName), str(self._obj.DeliveryMonth), int(self._obj.PositionBuyToday),
                int(self._obj.PositionBuyYesterday), int(self._obj.PositionBuy), int(self._obj.PositionSellToday), int(self._obj.PositionSellYesterday),
                int(self._obj.PositionSell), int(self._obj.PositionTotal), int(self._obj.CanceledOrderCount), int(self._obj.AddOrderCount),
                int(self._obj.SumTradeVolume), int(self._obj.SelfTradeCount), int(self._obj.ErrorOrderCount), int(self._obj.BuyOpenSum),
                int(self._obj.SellOpenSum), int(self._obj.BuySum), int(self._obj.SellSum), float(self._obj.BuySumPrice),
                float(self._obj.SellSumPrice), int(self._obj.UnTradeBuy), int(self._obj.UnTradeSell), int(self._obj.UntradeOpen),
                int(self._obj.UntradeBuyOpen), int(self._obj.UntradeSellOpen), int(self._obj.UntradeClose), int(self._obj.UntradeBuyClose),
                int(self._obj.UntradeSellClose), int(self._obj.BuyYdPosition), int(self._obj.SellYdPosition), float(self._obj.BuyPreSettlementPrice),
                float(self._obj.SellPreSettlementPrice), float(self._obj.LongAvgPrice), float(self._obj.ShortAvgPrice), float(self._obj.FrozenMargin),
                float(self._obj.LongFrozenMargin), float(self._obj.ShortFrozenMargin), float(self._obj.FrozenCommission), float(self._obj.OpenFrozenCommission),
                float(self._obj.CloseFrozenCommission), float(self._obj.CloseTodayFrozenCommission), float(self._obj.CloseProfit), float(self._obj.LongCloseProfit),
                float(self._obj.ShortCloseProfit), float(self._obj.CurrMargin), float(self._obj.LongCurrMargin), float(self._obj.ShortCurrMargin),
                float(self._obj.ShortBaseMargin), float(self._obj.ShortPosMargin), float(self._obj.Commission), float(self._obj.OpenCommission),
                float(self._obj.CloseCommission), float(self._obj.CloseTodayCommission), float(self._obj.PositionProfit), float(self._obj.LongPositionProfit),
                float(self._obj.ShortPositionProfit), float(self._obj.OrderCommission), float(self._obj.RoyaltyPositionProfit), float(self._obj.LongRoyaltyPositionProfit),
                float(self._obj.ShortRoyaltyPositionProfit), int(self._obj.LockBuyOpen), int(self._obj.LockBuyClose), int(self._obj.LockBuyCloseToday),
                int(self._obj.LockSellOpen), int(self._obj.LockSellClose), int(self._obj.LockSellCloseToday)]

    def tuple_to_obj(self, t):
        if len(t) >= 71:
            self._obj.InvestorID = t[0]
            self._obj.BrokerID = t[1]
            self._obj.ExchangeID = t[2]
            self._obj.ProductID = t[3]
            self._obj.InstrumentID = t[4]
            self._obj.InstrumentName = t[5]
            self._obj.DeliveryMonth = t[6]
            self._obj.PositionBuyToday = t[7]
            self._obj.PositionBuyYesterday = t[8]
            self._obj.PositionBuy = t[9]
            self._obj.PositionSellToday = t[10]
            self._obj.PositionSellYesterday = t[11]
            self._obj.PositionSell = t[12]
            self._obj.PositionTotal = t[13]
            self._obj.CanceledOrderCount = t[14]
            self._obj.AddOrderCount = t[15]
            self._obj.SumTradeVolume = t[16]
            self._obj.SelfTradeCount = t[17]
            self._obj.ErrorOrderCount = t[18]
            self._obj.BuyOpenSum = t[19]
            self._obj.SellOpenSum = t[20]
            self._obj.BuySum = t[21]
            self._obj.SellSum = t[22]
            self._obj.BuySumPrice = t[23]
            self._obj.SellSumPrice = t[24]
            self._obj.UnTradeBuy = t[25]
            self._obj.UnTradeSell = t[26]
            self._obj.UntradeOpen = t[27]
            self._obj.UntradeBuyOpen = t[28]
            self._obj.UntradeSellOpen = t[29]
            self._obj.UntradeClose = t[30]
            self._obj.UntradeBuyClose = t[31]
            self._obj.UntradeSellClose = t[32]
            self._obj.BuyYdPosition = t[33]
            self._obj.SellYdPosition = t[34]
            self._obj.BuyPreSettlementPrice = t[35]
            self._obj.SellPreSettlementPrice = t[36]
            self._obj.LongAvgPrice = t[37]
            self._obj.ShortAvgPrice = t[38]
            self._obj.FrozenMargin = t[39]
            self._obj.LongFrozenMargin = t[40]
            self._obj.ShortFrozenMargin = t[41]
            self._obj.FrozenCommission = t[42]
            self._obj.OpenFrozenCommission = t[43]
            self._obj.CloseFrozenCommission = t[44]
            self._obj.CloseTodayFrozenCommission = t[45]
            self._obj.CloseProfit = t[46]
            self._obj.LongCloseProfit = t[47]
            self._obj.ShortCloseProfit = t[48]
            self._obj.CurrMargin = t[49]
            self._obj.LongCurrMargin = t[50]
            self._obj.ShortCurrMargin = t[51]
            self._obj.ShortBaseMargin = t[52]
            self._obj.ShortPosMargin = t[53]
            self._obj.Commission = t[54]
            self._obj.OpenCommission = t[55]
            self._obj.CloseCommission = t[56]
            self._obj.CloseTodayCommission = t[57]
            self._obj.PositionProfit = t[58]
            self._obj.LongPositionProfit = t[59]
            self._obj.ShortPositionProfit = t[60]
            self._obj.OrderCommission = t[61]
            self._obj.RoyaltyPositionProfit = t[62]
            self._obj.LongRoyaltyPositionProfit = t[63]
            self._obj.ShortRoyaltyPositionProfit = t[64]
            self._obj.LockBuyOpen = t[65]
            self._obj.LockBuyClose = t[66]
            self._obj.LockBuyCloseToday = t[67]
            self._obj.LockSellOpen = t[68]
            self._obj.LockSellClose = t[69]
            self._obj.LockSellCloseToday = t[70]

            return True
        return False



from ..interface import IData
from ..packer.message_data_packer import MessageDataPacker


class MessageData(IData):
    def __init__(self, mid: int = -1, action_id: int = 1, request_id: int = 0, is_last: bool = True, rsp_success: bool = True, rsp_msg: str = '', udata: bytes = b'', compresstype: int = -1) -> None:
        super().__init__(MessageDataPacker(self))
        self._MID = mid
        self._ActionID: int = action_id
        self._RequestID: int = request_id
        self._IsLast: bool = is_last
        self._RspSuccess: bool = rsp_success
        self._RspMsg: str = rsp_msg
        self._UData: bytes = udata
        self._CompressType: int = compresstype

    @property
    def MID(self):
        return self._MID

    @MID.setter
    def MID(self, value: int):
        self._MID = value

    @property
    def ActionID(self):
        return self._ActionID

    @ActionID.setter
    def ActionID(self, value: int):
        self._ActionID = value

    @property
    def RequestID(self):
        return self._RequestID

    @RequestID.setter
    def RequestID(self, value: int):
        self._RequestID = value

    @property
    def IsLast(self):
        return self._IsLast

    @IsLast.setter
    def IsLast(self, value: bool):
        self._IsLast = value

    @property
    def RspSuccess(self):
        return self._RspSuccess

    @RspSuccess.setter
    def RspSuccess(self, value: bool):
        self._RspSuccess = value

    @property
    def RspMsg(self):
        return self._RspMsg

    @RspMsg.setter
    def RspMsg(self, value: str):
        self._RspMsg = value

    @property
    def UData(self):
        return self._UData

    @UData.setter
    def UData(self, value: bytes):
        self._UData = value

    @property
    def CompressType(self):
        return self._CompressType

    @CompressType.setter
    def CompressType(self, value: bytes):
        self._CompressType = value



from ...interface import IPacker


class SaveTickListParamDataPacker(IPacker):
    def __init__(self, obj) -> None:
        super().__init__(obj)

    def obj_to_tuple(self):
        ret = [
            str(self._obj.MarketName),
            str(self._obj.ExchangeID),
            str(self._obj.InstrumentID),
            str(self._obj.ProductID),
            str(self._obj.TradingDay),
            float(self._obj.PreClosePrice),
            float(self._obj.PreSettlementPrice),
            float(self._obj.PreOpenInterest),
            float(self._obj.UpperLimitPrice),
            float(self._obj.LowerLimitPrice),
            list(self._obj.ActionDayList),
            list(self._obj.TradingTimeList),
            list(self._obj.UpdateMillSecList),
            list(self._obj.LocalTimeList),
            list(self._obj.LastPriceList),
            list(self._obj.LastVolumeList),
            list(self._obj.BidPriceList),
            list(self._obj.BidVolumeList),
            list(self._obj.AskPriceList),
            list(self._obj.AskVolumeList),
            list(self._obj.AvgPriceList),
            list(self._obj.OpenPriceList),
            list(self._obj.HighPriceList),
            list(self._obj.LowerPriceList),
            list(self._obj.TotalTurnoverList),
            list(self._obj.TotalVolumeList),
            list(self._obj.OpenInterestList),
            list(self._obj.ClosePriceList),
            list(self._obj.SettlementPriceList),
            list(self._obj.TotalValueList)
        ]
        return ret

    def tuple_to_obj(self, t):
        if len(t) >= 30:
            self._obj.MarketName = t[0]
            self._obj.ExchangeID = t[1]
            self._obj.InstrumentID = t[2]
            self._obj.ProductID = t[3]
            self._obj.TradingDay = t[4]
            self._obj.PreClosePrice = t[5]
            self._obj.PreSettlementPrice = t[6]
            self._obj.PreOpenInterest = t[7]
            self._obj.UpperLimitPrice = t[8]
            self._obj.LowerLimitPrice = t[9]
            self._obj.ActionDayList = t[10]
            self._obj.TradingTimeList = t[11]
            self._obj.UpdateMillSecList = t[12]
            self._obj.LocalTimeList = t[13]
            self._obj.LastPriceList = t[14]
            self._obj.LastVolumeList = t[15]
            self._obj.BidPriceList = t[16]
            self._obj.BidVolumeList = t[17]
            self._obj.AskPriceList = t[18]
            self._obj.AskVolumeList = t[19]
            self._obj.AvgPriceList = t[20]
            self._obj.OpenPriceList = t[21]
            self._obj.HighPriceList = t[22]
            self._obj.LowerPriceList = t[23]
            self._obj.TotalTurnoverList = t[24]
            self._obj.TotalVolumeList = t[25]
            self._obj.OpenInterestList = t[26]
            self._obj.ClosePriceList = t[27]
            self._obj.SettlementPriceList = t[28]
            self._obj.TotalValueList = t[29]
            return True
        return False



from ...interface import IData
from ...packer.chart.plot_param_data_packer import PlotParamDataPacker


class PlotParamData(IData):
    def __init__(self, plot_index: int = 0, plot_name: str = '', plot_name_color: str = 'white', height: int = 100,
                 is_eqvalue_axis: bool = True, grid_style: int = 2, grid_color: str = "#323232", back_ground_color: str = "black",
                 border_color: str = "rgb(139,0,0)", border_width: int = 1, show_legend_item: bool = True, grid_x_style: int = 2, grid_x_color: str = '#323232'):
        """图块配置参数

        Args:
            plot_index (int, optional): 所属图块. Defaults to 0.
            plot_name (str, optional): 图块名称. Defaults to ''.
            plot_name_color (str, optional): 图块名称颜色. Defaults to 'white'.
            height (int, optional): 默认高度(比例). Defaults to 100.
            is_eqvalue_axis (bool, optional): 是否左右y轴相同显示. Defaults to True.
            grid_style (int, optional): 网格水平线样式.1.实线 2.线状虚线 3.点状虚线 4.点线间隔线 5.点点线间隔线 Defaults to 1 Defaults to 2.
            grid_color (str, optional): 网格水平线颜色. Defaults to "#323232".
            back_ground_color (str, optional): 背景色. Defaults to "black".
            border_color (str, optional): 边框颜色. Defaults to "rgb(139,0,0)".
            border_width (int, optional): 边框宽度. Defaults to 1.
            show_legend_item (bool, optional): 是否显示标签区域. Defaults to True.
            grid_x_style (int, optional): 网格垂直线样式.1.实线 2.线状虚线 3.点状虚线 4.点线间隔线 5.点点线间隔线 Defaults to 2.
            grid_x_color (str, optional): 网格垂直线颜色. Defaults to '#323232'.
        """
        super().__init__(PlotParamDataPacker(self))
        self._PlotIndex: int = plot_index
        self._Height: int = height
        self._IsEqValueAxis: bool = is_eqvalue_axis
        self._GridStyle: int = grid_style
        self._GridColor: str = grid_color
        self._BackGroundColor: str = back_ground_color
        self._BorderColor: str = border_color
        self._BorderWidth: int = border_width
        self._ShowLegendItem: bool = show_legend_item
        self._PlotName: str = plot_name
        self._PlotNameColor: str = plot_name_color
        self._GridXColor: str = grid_x_color
        self._GridXStyle: int = grid_x_style

    @property
    def PlotIndex(self):
        return self._PlotIndex

    @PlotIndex.setter
    def PlotIndex(self, value: int):
        self._PlotIndex = value

    @property
    def PlotName(self):
        return self._PlotName

    @PlotName.setter
    def PlotName(self, value: int):
        self._PlotName = value

    @property
    def PlotNameColor(self):
        return self._PlotNameColor

    @PlotNameColor.setter
    def PlotNameColor(self, value: int):
        self._PlotNameColor = value

    @property
    def Height(self):
        return self._Height

    @Height.setter
    def Height(self, value: int):
        self._Height = value

    @property
    def IsEqValueAxis(self):
        return self._IsEqValueAxis

    @IsEqValueAxis.setter
    def IsEqValueAxis(self, value: bool):
        self._IsEqValueAxis = value

    @property
    def GridStyle(self):
        return self._GridStyle

    @GridStyle.setter
    def GridStyle(self, value: int):
        self._GridStyle = value

    @property
    def GridColor(self):
        return self._GridColor

    @GridColor.setter
    def GridColor(self, value: str):
        self._GridColor = value

    @property
    def BackGroundColor(self):
        return self._BackGroundColor

    @BackGroundColor.setter
    def BackGroundColor(self, value: str):
        self._BackGroundColor = value

    @property
    def BorderColor(self):
        return self._BorderColor

    @BorderColor.setter
    def BorderColor(self, value: str):
        self._BorderColor = value

    @property
    def BorderWidth(self):
        return self._BorderWidth

    @BorderWidth.setter
    def BorderWidth(self, value: int):
        self._BorderWidth = value

    @property
    def ShowLegendItem(self):
        return self._ShowLegendItem

    @ShowLegendItem.setter
    def ShowLegendItem(self, value: bool):
        self._ShowLegendItem = value

    @property
    def GridXStyle(self):
        return self._GridXStyle

    @GridXStyle.setter
    def GridXStyle(self, value: int):
        self._GridXStyle = value

    @property
    def GridXColor(self):
        return self._GridXColor

    @GridXColor.setter
    def GridXColor(self, value: str):
        self._GridXColor = value
